// An ADD comment will be used to indicate variables and code added
// to the shader to increase functionality.
// The CHANGE comment will show changes made to demo shader code from 
// previous version.

#version 150 core

in vec3 position;

in vec3 color;

// ADD 
// Each vertex will have its own surface normal needed for computing lighting.
// Vertex normal
in vec3 normal;

uniform mat4 mvpMx;

// ADD
// We add a uniform variable to indicate transparency value: 1=opaque.
uniform float alpha;

// ADD
// Again, for lighting computations, we need the mv portion of the mvp matrix
// and its transposed inverse. 
// The mv matrix transforms the vertex into the eyeSpace sans perspective 
// projection.
uniform mat4 mvMx;

// ADD
// And the transpose of the inverse of the mv matrix is necessary for the
// general transformation of the normal vector. This is the general solution
// for anisotropic transformations. Not necessary here, but, again, the general
// solution.
uniform mat4 mvitMx;

// ADD
// A float masquerading as a boolean to indicate if lights are ON/OFF (1.0f, 0.0f).
uniform float lightson;

out vec4 Color;

// ADD
// Eye position of vertex.
out vec3 Position;

// ADD
// Normal at vertex - this is the direction at right angles to the surface of 
// which the vertex is part. This determines the orientation of the surface and
// how light will be reflected off of it.
out vec3 Normal;

// ADD
// Lights ON/OFF?
// a recomputed position vector, and the surface normal.
out float LightsOn;

// ADD
out vec4 lightPosition;

void main()
{

    // ADD
    // Backtransform hardcoded light position (50,75,0100) such that the light 
    // remains fixed when the object(s) rotate.
    lightPosition = inverse(mvpMx)*vec4(50, 75, -100, 1.0);

    // ADD
    // We transform the raw normal using the mvitMx, which provides for a more
    // general solution (anisotropic scaling) than absolutely necessary here.
    mat4 normalMatrix = mvitMx;

    // ADD
    // And make sure it is a unit-length vector.
    Normal = vec3(normalize(normalMatrix * vec4(normal,0)));

    // ADD
    // Where is the vertex with respect to the viewer, i.e., without perspective 
    // transformation.
    vec4 eyeCoords = mvMx * vec4(position, 1.0);

    // ADD
    // Eye position.
    Position = vec3(eyeCoords);

    gl_Position = mvpMx * vec4(position, 1.0);

    // CHANGE
    // We use the uniform alpha value for transparency so as not to require
    // an alpha for every vertex.
    Color = vec4(color, alpha);

    // ADD
    // Turn lighting ON/OFF.
    LightsOn = lightson;
}
