// An ADD comment will be used to indicate variables and code added
// to the shader to increase functionality.
// The CHANGE comment will show changes made to demo shader code from 
// the previous version.

#version 150 core

in vec4 Color; 

in float LightsOn;

// ADD
// A float acting as a boolean to turn texture usage ON/OFF for plotting axes.
// More generally, non-texture geometry should probably use their own shaders.
in float UseTexture;

in vec3 Position;

in vec3 Normal;

// ADD
// The interpolated texture coordinate of the incoming fragment.
in vec2 Texcoord;

// ADD 
// Sampler to extract texel color from texture.
uniform sampler2D Tex1;

in vec4 lightPosition;

out vec4 fragmentColor;

void main()
{
    // CHANGE
    // We actually ignore the color submitted for texture rendering,
    // but we use it if UseColor is <0.5. This allows us to use this
    // vertex/fragment shader pair for both texture rendering and
    // regular geometry drawing, i.e. the axes in this case. No other
    // code needs changing. Different from the previous fragment shader,
    // we use texColor for computations instead of the raw incoming color.
    vec3 texColor;
    if (UseTexture>0.5) {
        texColor = vec3(texture(Tex1, Texcoord));
    } else {
        texColor = vec3(Color);
    }

    if (LightsOn>0.5) {
	vec3 n = normalize(Normal);

	vec3 s = normalize(vec3(lightPosition) - Position);

	vec3 v = normalize(-Position);

	vec3 r = reflect(-s, n);

	vec3 h = normalize(v + s);

	vec3 Ka = vec3(0.3, 0.3, 0.3);
	vec3 La = vec3(texColor);


	vec3 Kd = vec3(0.75, 0.75, 0.75);

	vec3 Ld = vec3(texColor);
	
	vec3 Ks = vec3(0.2, 0.2, 0.2);
	vec3 Ls = vec3(1.0, 1.0, 1.0);
	
	vec3 ambient = La * Ka;
	float sDotN = max( dot(s, n), 0.0);
	vec3 diffuse = Ld * Kd * sDotN;
	
	vec3 spec = vec3(0.0);	
	float Shininess = 25;
	if (sDotN > 0.0)
	{
		spec = Ls * Ks * pow(max(dot(h,n), 0.0), Shininess);
	}

	vec3 res = ambient + diffuse + spec;
	
	fragmentColor = vec4(res, Color.a); 
    } else {
        fragmentColor = vec4(texColor, Color.a); 
    }
}
