// An ADD comment will be used to indicate variables and code added
// to the shader to increase functionality.
// The CHANGE comment will show changes made to demo shader code from 
// previous version.

#version 150 core

in vec3 position;

in vec3 color;

in vec3 normal;

// ADD
// Here we receive the texture coordinate associated with the vertex coordinate.
// The vertex coordinate is associated with the texture coordinate, and GLSL
// handles the interpolation.
in vec2 texcoord;

uniform mat4 mvpMx;

uniform float alpha;

uniform mat4 mvMx;

uniform mat4 mvitMx;

uniform float lightson;

// ADD 
// A float masquerading as a boolean to bypass texture mapping when 
// displaying axes. More generally, non-texture geometry should probably use 
// their own shaders.
uniform float useTexture;

out vec4 Color;

out vec3 Position;

out vec3 Normal;

out float LightsOn;

// ADD
// Use texture or not (for displaying axes).
out float UseTexture;

out vec4 lightPosition;

// ADD
// We pass along the texture coordinate associated with the vertex coordinate.
out vec2 Texcoord;

void main()
{
    // CHANGE
    lightPosition = inverse(mvpMx)*vec4(50, 75, -100, 1.0);

    mat4 normalMatrix = mvitMx;

    Normal = vec3(normalize(normalMatrix * vec4(normal,0)));

    vec4 eyeCoords = mvMx * vec4(position, 1.0);

    Position = vec3(eyeCoords);

    gl_Position = mvpMx * vec4(position, 1.0);

    Color = vec4(color, alpha);

    LightsOn = lightson;

    // ADD
    UseTexture = useTexture;

    // ADD
    // Send the frag shader the texture coordinate, now associated with the
    // transformed vertex coordinate.
    Texcoord = texcoord;
}
